typedef Repartition = {
	var nbGroupesClasse : Int;
	var nbGroupesReduits : Int;
	var cout : Int;
}

class Specialite {
	var estSpecialiteTerminale : Bool;
	public var libelle(default, null) : String;
	public var effectif : Int; //OPTIMISATION > SECURITE
	public var capacite(default, null) : Int;

	public var effectifClasse(default, null) : Int; //effectif maximum d'une classe entière
	public var effectifReduit(default, null) : Int; //effectif maximum d'une classe non dédoublée / groupe de travaux pratiques (ou 0 si on n'en veut pas)
	public var maxGroupes(default, null) : Int; //nombre maximum de groupes (classe entière ou effectif réduit - ou 0 si pas de limite)

	/* L'algorithme favorise les classes non dédoublées pour faciliter les affectations (avec plus de groupes),
	 * et simplifier la création de l'EDT (il n'est pas utile de placer 2 heures ailleurs pour le second groupe).
	 * Il n'existe pas de solution moins coûteuse en nombre d'heures que celle retenue. */
	public var nbGroupesClasse(default, null) : Int;
	public var nbGroupesReduits(default, null) : Int;
	public var priorite : Int;

	//static inline var EFFECTIF_MINIMUM : Int = 12;
	static inline var NB_HEURES_DEDOUBLEES_PREMIERE : Int = 2;
	static inline var NB_HEURES_DEDOUBLEES_TERMINALE : Int = 2;

	public var coutEffectifClasse(get, null) : Int;
	function get_coutEffectifClasse() : Int {
		var cout : Int = 0;
		if (estSpecialiteTerminale) {
			if (effectifReduit == 0) {
				cout = 6;
			} else {
				cout = 6 + NB_HEURES_DEDOUBLEES_TERMINALE;
			}
		} else {
			if (effectifReduit == 0) {
				cout = 4;
			} else {
				cout = 4 + NB_HEURES_DEDOUBLEES_PREMIERE;
			}
		}
		return cout;
	}
	public var coutEffectifReduit(get, null) : Int;
	function get_coutEffectifReduit() : Int {
		return estSpecialiteTerminale ? 6 : 4;
	}

	public function new(libelle : String, effectifClasse : Int = 35, effectifReduit : Int = 0, maxGroupes : Int = -1) {
		this.libelle = libelle;
		this.effectif = 0;
		this.effectifClasse = effectifClasse;
		this.effectifReduit = effectifReduit;
		this.maxGroupes = maxGroupes;
		this.capacite = 0;
		this.priorite = 0;
	}

	public function creerGroupes(pourTerminale: Bool) {
		this.estSpecialiteTerminale = pourTerminale; //ne peut être fait dans le constructeur en raison de l'initialisation à partir du fichier de paramètres
		//~ if (effectif < EFFECTIF_MINIMUM) {
			//~ nbGroupesReduits = 0;
			//~ nbGroupesClasse = 0;
		//~ } else {
			if (effectifReduit == 0) { //on ne prévoit pas de classe non dédoublée
				nbGroupesReduits = 0;
				nbGroupesClasse = Math.ceil(effectif / effectifClasse);
			} else {
				var solutions = optimumGroupes(effectif);
				nbGroupesReduits = solutions[0].nbGroupesReduits;
				nbGroupesClasse = solutions[0].nbGroupesClasse;
			}
			if (maxGroupes != -1 && nbGroupesClasse + nbGroupesReduits > maxGroupes) {
				nbGroupesReduits = 0;
				nbGroupesClasse = maxGroupes;
			}
			capacite = nbGroupesReduits * effectifReduit + nbGroupesClasse * effectifClasse;
		//~ }
	}

	function optimumGroupes(nbEleves : Int) : Array<Repartition> {
		var solutions : Array<Repartition> = new Array();
		var maxNbGroupesClasse = Math.ceil(nbEleves / effectifClasse);
		var maxNbGroupesReduits = Math.ceil(nbEleves / effectifReduit);

		var coutMin : Int = maxNbGroupesClasse * coutEffectifClasse + maxNbGroupesReduits * coutEffectifReduit;
		for (nbGrpCls in 0...maxNbGroupesClasse + 1) {
			for (nbGrpRed in 0...maxNbGroupesReduits + 1) {
				var cout = nbGrpCls * coutEffectifClasse + nbGrpRed * coutEffectifReduit;
				if (nbGrpCls * effectifClasse + nbGrpRed * effectifReduit >= nbEleves) {
					if (cout < coutMin) {
						coutMin = cout;
						solutions = new Array();
					}
					if (cout == coutMin) {
						solutions.push({ nbGroupesClasse : nbGrpCls, nbGroupesReduits : nbGrpRed, cout : cout });
					}
				}
			}
		}

		return solutions;
	}
}
